#!/usr/local/bin/perl 
#use strict;

# we use Date-Calc-4.2.tar.gz from CPAN
use Date::Calc qw(Delta_Days check_date Today);

#- Configurable Variables ---------------------------------------------------#

 $swish = '/usr/local/bin/swish-e';

 $swishcgi = '/cgi-bin/lookup.cgi';

 %parts = (
	'all.swish' => 'All Sections',
	'linux.swish' => 'Linux Documentation',
	'apache.swish' => 'Apache Documentation'
);

 %replace = (
	
	'/usr/doc' => '/docs',
	'/home/httpd/html' => '/apache'
);
# for complete urls instead of absolute pathname we need to
# substitute some, depending on swish.conf
# $docroot = '/home/web';
# $urlprefix = '';
# $pathcut = '/home/web/fci/www/fci/library/';

# $urlprefix = 'http://www.fci.nl';
 $helpurl = '/help.html';

#- Standard Stuff you won't need to edit -------------------------------------#

my (@query,$query,$index,$results);
my ($deltabefore,$deltaafter,$deltabetween);
my ($year,$month,$day)	= Today();

#- Main Program -------------------------------------------------------------#


&parse_form_data (*FORM);
# generate the select with the sections
$FORM{'index'} ? $index = $FORM{'index'} : $index = 'all.swish';
 $section = 'All Sections';
 $select = &makeselect;
# if checkboxes are checked check them again in this instance
# make sure they are checked so state is maintained maximally
 $aftercheck = $FORM{'after'};
 $beforecheck = $FORM{'before'};
 $after = &makedatemenu(1,1);
 $before = &makedatemenu(2,0);
 
if ($FORM{'search'} =~ /\w+/) {
   push (@query, $FORM{'search'});
}

$query = join (" and ", @query);
$results = $FORM{'results'};
# Sanity checks
&checkdates;

if ($query) {
   $index = $FORM{'index'} ;
   &search_parse;
   &output;
}else{
   &html_header("Swish-E Search");
   &html_trailer;
}

#- Subroutines ---------------------------------------------------------------#

#Subroutine for checking and parsing the incoming form data.

sub parse_form_data {

local (*FORM_DATA) = @_;
local ($request_method, $query_string, @key_value_pairs, $key_value, $key, $value);

my $request_method = $ENV{'REQUEST_METHOD'};

if ($request_method eq 'GET') {
   $query_string = $ENV{'QUERY_STRING'};
} elsif ($request_method eq 'POST') {
   if($ENV{'CONTENT_LENGTH'} > 64000){
	  &search_error("Request to receive too much data: $ENV{'CONTENT_LENGTH'}  bytes\n");
   }
   read (STDIN, $query_string, $ENV{'CONTENT_LENGTH'});
} else {
   &search_error ("Forms must use either GET or POST.");
}

@key_value_pairs = split (/&/, $query_string);

foreach $key_value (@key_value_pairs) {
   ($key,  $value) = split (/=/, $key_value);
   $value =~ tr/+/ /;
   $value =~ s/%(..)/pack('c',hex($1))/eg;
   if (defined ($FORM_DATA{$key})) {
      $FORM_DATA{$key} = join ("\0", $FORM_DATA{$key}, $value);
   } else {
      $FORM_DATA{$key} = $value;
   }
}

}

#Subroutine for constructing the Swish-E search request and formating the results.

sub search_parse {
	
	$count=0;
	
	# Go via forked child to avoid shell escapes, report errors of the child
	open SWISH, '-|' or exec $swish, '-w', $query, '-m', $results, '-f', $index or exec 'echo', "err: $!";
	
	while (<SWISH>) {
	   chop;
	
	   if ($_ eq "err: no results") 
	   {&search_error("Your Search in <i>'$section'</i> for <strong>$query</strong> returned <B>no</B> items");}
	
	   if ($_ eq "err: could not open index file") 
	   {&search_error("Could not open SWISH Index File $index");}
	   
	   if ($_ eq "err: no search words specified") 
	   {&search_error("Please Enter at least one Search Word");}
	
	   if ($_ eq "err: a word is too common") 
	   {&search_error("One of your search terms is too common, please try again");}
	  
	   if  (/err: (.*)/) 
	   {&search_error("Swish error: $1");}	
	
	   next if /^\D/;
	   $count++;
	   push(@results, $_);
	
	}
	close SWISH;
}

# generate the pulldown menu with the sections of the archive
sub makeselect{
	$html = '<SELECT NAME="index">';
	foreach $key (sort keys %parts){
		if($key eq $FORM{'index'}){
			$html .= qq|<OPTION SELECTED VALUE="$key">$parts{$key}|;
			$section = $parts{$key};
		}else{
			$html .= qq|<OPTION VALUE="$key">$parts{$key}|;
		}
	}
	$html .= '</SELECT>';
	return $html;
}

# check if filled in dates exist
sub checkdates{
	unless($FORM{'year1'}){ $FORM{'year1'} = $year;}
	unless($FORM{'month2'}){ $FORM{'month1'} = $month;}
	unless($FORM{'day1'}){  $FORM{'day1'} = $day;}
	
	unless($FORM{'year2'}){  $FORM{'year2'} = $year;}
	unless($FORM{'month2'}){  $FORM{'month2'} = $month;}
	unless($FORM{'day2'}){  $FORM{'day2'} = $day;}	
	# sanity check on the dates
	if($FORM{'after'}){
		unless (check_date(($FORM{'year1'}, $FORM{'month1'}, $FORM{'day1'}))){
			&search_error ("Date 'After' not correct");
		}
		$deltaafter = Delta_Days($FORM{'year1'}, $FORM{'month1'}, $FORM{'day1'},
				   "$year", "$month", "$day");
		if($deltaafter < 0){ &search_error("Future unknown"); }
	}
	if($FORM{'before'}){	
		unless (check_date(($FORM{'year2'}, $FORM{'month2'}, $FORM{'day2'}))){
			&search_error ("Date 'Before' not correct");
		}
		$deltabefore = Delta_Days($FORM{'year2'}, $FORM{'month2'}, $FORM{'day2'},
				   "$year", "$month", "$day");
	}	
	
	if($FORM{'before'}&&$FORM{'after'}){
		$deltabetween = Delta_Days($FORM{'year1'}, $FORM{'month1'}, $FORM{'day1'},
				   $FORM{'year2'}, $FORM{'month2'}, $FORM{'day2'});
		unless($deltabetween >= 0){			   
			&search_error("Dates are excluding all files, something wrong");
		}
	}	
}
	
sub makedatemenu{
	my  @days	= ('01'..'31');
	my  @months	= ('01'..'12');

	@years	= ($year-5,$year-4,$year-3,$year-2,$year-1,$year);
	
	$html = qq|<SELECT NAME="day$_[0]">|;
	foreach $regel (@days) {
		if ($regel eq $FORM{"day$_[0]"}) {
			$html .= "<OPTION SELECTED VALUE=\"$regel\">$regel\n";
		} else {
			$html .= "<OPTION VALUE=\"$regel\">$regel\n";
		}
	}
	$html .= '</SELECT>';
	
	$html .= qq|<SELECT NAME="month$_[0]">|;
	foreach $regel (@months) {
		if ($regel eq $FORM{"month$_[0]"}) {
			$html .= "<OPTION SELECTED VALUE=\"$regel\">$regel\n";
		} else {
			$html .= "<OPTION VALUE=\"$regel\">$regel\n";
		}
	}
	$html .= '</SELECT>';
	
	$html .= qq|<SELECT NAME="year$_[0]">|;
	foreach $regel (@years) {	
		if ($regel eq $FORM{"year$_[0]"}) {
			$html .= "<OPTION SELECTED VALUE=\"$regel\">$regel\n";
		} else {
			$html .= "<OPTION VALUE=\"$regel\">$regel\n";
		}
	}
	$html .= '</SELECT>';
	return $html;
}


sub output{
	my ($out,$url,$stringone,$title,$filesize,$rank,$file,$url,$label);
	&html_header("Search Results");
	
	print <<EOT;
	<TABLE BORDER="0" WIDTH=550>
	<TR><TD>Your Search in <i>'$section'</i> for <strong>$query</strong> returned the following items, listed in order of computed relevance,
	</P>
	</TR>
	<TR><TD><HR NOSHADE WIDTH="100%" SIZE="4"></TR>
EOT

	foreach (@results) {
	   ($stringone, $title, $filesize) = split(/\"/, $_);
	   ($rank, $file) = split(/ /, $stringone);
	   $url = $file;
	   $label = $file;
	   #$url =~ s/$docroot/$urlprefix/;
	   #$label =~ s/$pathcut//;
#	$line = qq| <TR><TD><B><A HREF="$url">$label</A></B></TD></TR>
#<TR><TD>$filesize b. <B>$title</B> <I>Score: $rank</I></TD></TR>|; 
#	print $line;

	# translate pathnames to urls according to %replace rules
	foreach $path (keys %replace){
		$url =~ s/$path/$replace{$path}/;
	}

       $line = qq|<TR><TD><B><A HREF=\"$url\">$label</A></B></TD></TR><TR><TD><NOBR><B>#MODTIME#</B></NOBR>$filesize b. <B>$title</B> <I>Score: $rank</I></TD><TR>\n<TR><TD><HR NOSHADE WIDTH="100%"></TD></TR>|;         
	   
	   $out = &checkfile($file,$line);
	   
	   print $out;	   
	}
	print "</TABLE>\n";
	&html_trailer;	
}

sub checkfile {
   
  my ($daysbefore,$daysafter,$string,$tmpstring);
   my @DoW = ('Sun','Mon','Tue','Wed','Thu','Fri','Sat');
   my @MoY = ('Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec');

   local($[) = 0;
   local ($sec, $min, $hour, $mday, $mon, $fyear, $wday, $yday, $isdst);
	
   # Get the modification date of this file (IF IT EXISTS!!!)
    local(@filestat) = stat($_[0]);
    local ($time) = $filestat[9];
    local ($size) = $filestat[7];
    
    
    # due to url-translations we need to know again
    $string = $_[1];
    if($size){
    	$string =~ s/#SIZE#/$size b./;
	}else{
		$string =~ s/#SIZE#/size n.a./;
	}	
	if (!$time){ 
		$string =~ s/#MODTIME#/time n.a./;
	}
	
   # Determine what time zone is in effect.
   # Use GMT if TZ is defined as null, local time if TZ undefined.
   # There's no portable way to find the system default timezone.

   $TZ = defined($ENV{'TZ'}) ? ( $ENV{'TZ'} ? $ENV{'TZ'} : 'GMT' ) : '';
   ($sec, $min, $hour, $mday, $mon, $fyear, $wday, $yday, $isdst) =
        ($TZ eq 'GMT') ? gmtime($time) : localtime($time);

   # Hack to deal with 'PST8PDT' format of TZ
   # Note that this can't deal with all the esoteric forms, but it
   # does recognize the most common: [:]STDoff[DST[off][,rule]]

   if($TZ=~/^([^:\d+\-,]{3,})([+-]?\d{1,2}(:\d{1,2}){0,2})([^\d+\-,]{3,})?/){
       $TZ = $isdst ? $4 : $1;
   }
   $TZ .= ' ' unless $TZ eq '';

   $fyear += 1900;
   
   
   $tmpstring = sprintf("%s %2d %s %4d %2d:%02d:%02d %s\n",
      $DoW[$wday], $mday, $MoY[$mon], $fyear,$hour, $min, $sec, $TZ);
      
   $mon +=1;
   $daysafter = Delta_Days($FORM{'year1'}, $FORM{'month1'}, $FORM{'day1'},$fyear, $mon, $mday);
   $daysbefore = Delta_Days($fyear, $mon, $mday,$FORM{'year2'}, $FORM{'month2'}, $FORM{'day2'});
  
 
   # $string =~ s/#MODTIME#/$fyear-$mon-$mday : A $daysafter : B $daysbefore/;
	
	$string =~ s/#MODTIME#/$tmpstring/;
   	# if checkboxes are checked both dates should be checked
   	 
   	if($FORM{'after'}){
   		if ($daysafter < 0){
   			return '';
   		}elsif($FORM{'before'}){
			if ($daysbefore < 0){
				return '';
			}else{
				return $string;
			}
   		}else{
   			return $string;
   		}
   	}elsif($FORM{'before'} && defined($daysbefore)){
		if ($daysbefore < 0){
			return '';
		}else{
			return $string;
		}
		
   	}else{
   		return $string;
   	}
}

#Subroutine for print a generic HTML header.

sub html_header {

$document_title = $_[0];

print<<HTML_Header;
Content-type: text/html

<HTML>
<HEAD>
<TITLE>$document_title</TITLE>
<!-- Site built by Antraciet http://www.antraciet.nl -->
<META NAME="Description" CONTENT="The Searchengine of Unix Documentation">
<META NAME="Keywords" CONTENT="Search engine, search, lookup, query, index">
<META HTTP-EQUIV="Author" CONTENT="Bas Meijer">
<META HTTP-EQUIV="Organization" CONTENT="Antraciet">
</HEAD>
<BODY TEXT="#000066" BGCOLOR="#FFFFFF" LINK="#0033CC" VLINK="#CC0000"
ALINK="#6666CC">
<BLOCKQUOTE>
<TABLE BORDER="0" WIDTH="550">
<TR><TD><FONT SIZE="5">Searchengine</FONT></TD></TR>
<TR><TD>Search (sections of) this website using keywords. You can combine keywords with
<b>and</b>, <b>or</b> &amp; <b>not</b>, you can group clauses with parentheses <b>(</b> <b>)</b>.
Use <b>*</b> as an end of word wildcard. <A HREF="$helpurl">More Help</A>.
<FORM ACTION="$swishcgi" method="GET">
<TABLE BORDER="0" CELLSPACING="0" CELLPADDING="0" ALIGN="left">
	<TR VALIGN="top">
		<TD><B>Keywords</B>:</TD><TD>
		<INPUT TYPE="TEXT" NAME="search" SIZE="35" VALUE="$query">
		<INPUT TYPE="HIDDEN" name="results" value="0"></TD>
	</TR>
	<TR VALIGN="top">
		<TD><B>in section</B>: </TD><TD>$select</TD> 
	</TR>
	<TR VALIGN="top">
		<TD><INPUT TYPE="CHECKBOX" NAME="after" $aftercheck VALUE="CHECKED"><B>on/after</B>: </TD><TD>$after</TD>
	</TR>
	<TR VALIGN="top">
		<TD><INPUT TYPE="CHECKBOX" NAME="before" $beforecheck VALUE="CHECKED"><B>on/before</B>: </TD><TD>$before</TD>
	</TR>
	<TR VALIGN="top">
		<TD COLSPAN=2 ALIGN=CENTER><INPUT TYPE="SUBMIT" VALUE="Start Search"></TD>
	</TR>
</TABLE>
</FORM>
</TD></TR>
<TR><TD COLSPAN=2>
<HR NOSHADE WIDTH="100%" SIZE="4">
</TD></TR>
</TABLE>
HTML_Header
}

#Subroutine for printing a generic HTML trailer.

sub html_trailer {
print <<HTML_Trailer;
</BODY></HTML>
HTML_Trailer
exit;
}

#Subroutine for printing error messages.

sub search_error {
 $error_message = $_[0];

&html_header("Swish-E Search Error");
print qq|<P>\n$error_message</P>\n|;
&html_trailer;      

}

